#!/usr/bin/env python3
"""
Copyright (c) 2013 Adafruit
Author of Py-Wrapper version: Justin Cooper
Author of Full Python version: Mark Yoder, Joshua Key, and Eric Morse

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
"""PWM functionality of a BeagleBone using Python."""
import glob
import time

t_NULL=0
t_module = 0
t_sysfs  = 1
t_index  = 2
t_muxmode= 3
t_path   = 4
t_name   = 5
t_chip   = 6
t_addr   = 7
t_key    = 8

pwm_table = [
  [ "ehrpwm2", 6, 1, 4, "ehrpwm.2:1", "EHRPWM2B", "48304000", "48304200", "P8_13"],
  [ "ehrpwm2", 5, 0, 4, "ehrpwm.2:0", "EHRPWM2A", "48304000", "48304200", "P8_19"],
  [ "ehrpwm1", 4, 1, 2, "ehrpwm.1:1", "EHRPWM1B", "48302000", "48302200", "P8_34"],
  [ "ehrpwm1", 3, 0, 2, "ehrpwm.1:0", "EHRPWM1A", "48302000", "48302200", "P8_36"],
  [ "ehrpwm2", 5, 0, 3, "ehrpwm.2:0", "EHRPWM2A", "48304000", "48304200", "P8_45"],
  [ "ehrpwm2", 6, 1, 3, "ehrpwm.2:1", "EHRPWM2B", "48304000", "48304200", "P8_46"],
  [ "ehrpwm1", 3, 0, 6, "ehrpwm.1:0", "EHRPWM1A", "48302000", "48302200", "P9_14"],
  [ "ehrpwm1", 4, 1, 6, "ehrpwm.1:1", "EHRPWM1B", "48302000", "48302200", "P9_16"],
  [ "ehrpwm0", 1, 1, 3, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P9_21"],
  [ "ehrpwm0", 0, 0, 3, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P9_22"],
  [   "ecap2", 7, 0, 4, "ecap.2",     "ECAPPWM2", "48304000", "48304100", "P9_28"],
  [ "ehrpwm0", 1, 1, 1, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P9_29"],
  [ "ehrpwm0", 0, 0, 1, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P9_31"],
  [   "ecap0", 2, 0, 0, "ecap.0",     "ECAPPWM0", "48300000", "48300100", "P9_42"],
  [ "timer4", 0, 0, 2, "", "", "", "dmtimer-pwm-4", "P8_7" ],
  [ "timer7", 0, 0, 2, "", "", "", "dmtimer-pwm-7", "P8_8" ],
  [ "timer5", 0, 0, 2, "", "", "", "dmtimer-pwm-5", "P8_9" ],
  [ "timer6", 0, 0, 2, "", "", "", "dmtimer-pwm-6", "P8_10" ],
  [ "ehrpwm0", 0, 0, 1, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P1_8"],
  [ "ehrpwm0", 0, 0, 1, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P1_36"],
  [ "ehrpwm0", 1, 1, 1, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P1_10"],
  [ "ehrpwm0", 1, 1, 1, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P1_33"],
  [ "ehrpwm1", 3, 0, 6, "ehrpwm.1:0", "EHRPWM1A", "48302000", "48302200", "P2_1"],
  [ "ehrpwm2", 6, 1, 3, "ehrpwm.2:1", "EHRPWM2B", "48304000", "48304200", "P2_3"],
  [ "timer7", 0, 0, 4, "", "", "", "dmtimer-pwm-7", "P1_20" ],
  [ "timer6", 0, 0, 1, "", "", "", "dmtimer-pwm-6", "P1_26" ],
  [ "timer5", 0, 0, 1, "", "", "", "dmtimer-pwm-5", "P1_28" ],
  [ "timer7", 0, 0, 5, "", "", "", "dmtimer-pwm-7", "P2_27" ],
  [ "timer4", 0, 0, 2, "", "", "", "dmtimer-pwm-4", "P2_31" ],
]

# This is needed to access the timer pwm's
# uboot_overlay_addr0=/lib/firmware/BB-PWM-TIMER-P8.07.dtbo
# uboot_overlay_addr1=/lib/firmware/BB-PWM-TIMER-P8.08.dtbo
# uboot_overlay_addr2=/lib/firmware/BB-PWM-TIMER-P8.09.dtbo
# uboot_overlay_addr3=/lib/firmware/BB-PWM-TIMER-P8.10.dtbo
# https://github.com/adafruit/adafruit-beaglebone-io-python/issues/229

# print(pwm_table[2][t_key])
# print(len(pwm_table))

def get_pwm_key(channel):
    for i in range(len(pwm_table)):
        if pwm_table[i][t_key] == channel:
            return pwm_table[i]
    print(channel + ': Not Found')
    return None

def get_pwm_path(channel):
    x = get_pwm_key(channel)
    if x == None:
        return None
    pwmPath =  glob.glob("/sys/devices/platform/ocp/" + x[t_chip] + ".epwmss/" 
            + x[t_addr] + ".pwm/pwm/*")[0]
    # print(pwmPath)
    return [pwmPath, x[t_index]]

def start(channel, duty, freq=2000, polarity=0):
    """Set up and start the PWM channel.
    
    channel can be in the form of 'P8_10', or 'EHRPWM2A'"""
    print(channel)
    path = get_pwm_path(channel)
    if path == None:
        return None
    # /sys/devices/platform/ocp/48302000.epwmss/48302200.pwm/pwm/pwmchip4/pwm-4:0
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])
    # print(pathpwm)

    period_ns = 1e9 / freq
    duty_ns = period_ns * (duty / 100.0)

    # export
    try:
        fd = open(path[0] + "/export", 'w')
        fd.write(str(path[1])) 
        fd.close()
    except:
        pass
    time.sleep(0.05)    # Give export a chance
    
    set_pin_mode(channel, 'pwm')
    
    # Duty Cycle - Set to 0 so period can be changed
    try:
        fd = open(pathpwm + "/duty_cycle", 'w')
        fd.write('0')
        fd.close()
    except:
        pass

    # Period
    fd = open(pathpwm + "/period", 'w')
    fd.write(str(int(period_ns)))
    fd.close()

    # Duty Cycle
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write(str(int(duty_ns)))
    fd.close()

    # Enable
    fd = open(pathpwm + "/enable", 'w')
    fd.write('1')
    fd.close()
    
    return 'Started'

def set_frequency(channel, freq):
    """Change the frequency
    
    frequency - frequency in Hz (freq > 0.0)"""
    path = get_pwm_path(channel)
    # /sys/devices/platform/ocp/48302000.epwmss/48302200.pwm/pwm/pwmchip4/pwm-4:0
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])

    # compute current duty cycle
    fd = open(pathpwm + "/duty_cycle", 'r')
    duty_cycle_ns = int(fd.read()[:-1] )       # Remove \n at end
    fd.close()
    
    fd = open(pathpwm + "/period", 'r')
    period_ns = int(fd.read()[:-1])
    fd.close()
    
    duty_cycle = duty_cycle_ns/period_ns          # compute current duty cycle as fraction
    period_ns = 1e9 / freq                  # compute new period
    duty_cycle_ns = period_ns*duty_cycle # compute new duty cycle as fraction of period

    # print('duty_cycle: ' + str(duty_cycle))
    # print('period_ns: ' + str(period_ns))
    # print('duty_cycle_ns: ' + str(duty_cycle_ns))

    # Duty Cycle - Set to 0
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write('0')
    fd.close()
    
    # Period
    fd = open(pathpwm + "/period", 'w')
    fd.write(str(int(period_ns)))
    fd.close()

    # Duty Cycle
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write(str(int(duty_cycle_ns)))
    fd.close()

def set_duty_cycle(channel, duty):
    """Change the duty cycle.
    
    dutycycle - between 0.0 and 100.0"""
    path = get_pwm_path(channel)
    # /sys/devices/platform/ocp/48302000.epwmss/48302200.pwm/pwm/pwmchip4/pwm-4:0
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])

    fd = open(pathpwm + "/period", 'r')
    period_ns = int(fd.read()[:-1])
    fd.close()

    duty_cycle_ns = duty/100 * period_ns
    # Duty Cycle
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write(str(int(duty_cycle_ns)))
    fd.close()
    
def stop(channel):
    """Stop the PWM channel.
    
    channel can be in the form of 'P8_10', or 'EHRPWM2A'"""
    path = get_pwm_path(channel)
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])
   
    # Disable
    fd = open(pathpwm + "/enable", 'w')
    fd.write('0')
    fd.close()

    # unexport
    fd = open(path[0] + "/unexport", 'w')
    fd.write(str(path[1])) 
    fd.close()
    
    set_pin_mode(channel, 'gpio')

def set_pin_mode(channel, mode):
    path = '/sys/devices/platform/ocp/ocp:' + channel + '_pinmux/state'
    fd = open(path, 'w')
    fd.write(mode)
    fd.close()

    return path



#!/usr/bin/env python3
"""gpiod-based Pinmux functionality of a BeagleBone using Python."""
import gpiod
import sys
from collections import defaultdict

def def_value():
	return "<INVALID>"

BBIO_ERR = defaultdict(def_value)

BBIO_ERR = {
	"BBIO_OK" : "OK",
	"BBIO_ACCESS" : "ACCESS",
	"BBIO_SYSFS" : "SYSFS",
	"BBIO_CAPE" : "CAPE",
	"BBIO_INVARG": "INVARG",
	"BBIO_MEM": "MEMORY",
	"BBIO_GEN": "GENERAL"
}

# function to set pin mode
def set_pin_mode(channel, mode):
	path = '/sys/devices/platform/ocp/ocp:' + channel + '_pinmux/state'
	fd = open(path, 'w')
	fd.write(mode)
	fd.close()

	return path



#!/usr/bin/env python3
"""
Copyright (c) 2013 Adafruit

Original RPi.GPIO Py-Wrapper Author Ben Croston
Modified for BBIO Py-Wrapper Author Justin Cooper
Authors of Full Python Implementation: Mark Yoder, Joshua Key, and Eric Morse

This file incorporates work covered by the following copyright and 
permission notice, all modified code adopts the original license:

Copyright (c) 2013 Ben Croston

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
"""gpiod-based GPIO functionality of a BeagleBone using Python."""
import gpiod
import sys
#import multiprocessing
from multiprocessing import Pipe, Process
# import select
import time

ALT0 = 4
BOTH = 3
FALLING = 2
HIGH = 1
IN = 0
LOW = 0
OUT = 1
PUD_DOWN = 1
PUD_OFF = 0
PUD_UP = 2
RISING = 1
VERSION = '0.0.0'
threads=[]
ports={}        # Dictionary of channel/line pairs that are open
CONSUMER='GPIOmay'
parent_conn = []
gChannel = []
gChannelNames = []

# Table generated based on https://github.com/jadonk/bonescript/blob/master/src/bone.js
table = [
  [ "USR0", "USR0", 53, -1, -1, "GPMC_A5"],
  [ "USR1", "USR1", 54, -1, -1, "GPMC_A6"],
  [ "USR2", "USR2", 55, -1, -1, "GPMC_A7"],
  [ "USR3", "USR3", 56, -1, -1, "GPMC_A8"],
  [ "DGND", "P8_1", 0, -1, -1, "unused"],
  [ "DGND", "P8_2", 0, -1, -1, "unused"],
  [ "GPIO1_6", "P8_3", 38, -1, -1, "GPMC_AD6"],
  [ "GPIO1_7", "P8_4", 39, -1, -1, "GPMC_AD7"],
  [ "GPIO1_2", "P8_5", 34, -1, -1, "GPMC_AD2"],
  [ "GPIO1_3", "P8_6", 35, -1, -1, "GPMC_AD3"],
  [ "TIMER4", "P8_7", 66, 2, -1, "GPMC_ADVN_ALE"],
  [ "TIMER7", "P8_8", 67, 2, -1, "GPMC_OEN_REN"],
  [ "TIMER5", "P8_9", 69, 2, -1, "GPMC_BEON_CLE"],
  [ "TIMER6", "P8_10", 68, 2, -1, "GPMC_WEN"],
  [ "GPIO1_13", "P8_11", 45, -1, -1, "GPMC_AD13"],
  [ "GPIO1_12", "P8_12", 44, -1, -1, "GPMC_AD12"],
  [ "EHRPWM2B", "P8_13", 23, 4, -1, "GPMC_AD9"],
  [ "GPIO0_26", "P8_14", 26, -1, -1, "GPMC_AD10"],
  [ "GPIO1_15", "P8_15", 47, -1, -1, "GPMC_AD15"],
  [ "GPIO1_14", "P8_16", 46, -1, -1, "GPMC_AD14"],
  [ "GPIO0_27", "P8_17", 27, -1, -1, "GPMC_AD11"],
  [ "GPIO2_1", "P8_18", 65, -1, -1, "GPMC_DK_MUX0"],
  [ "EHRPWM2A", "P8_19", 22, 4, -1, "GPMC_AD8"],
  [ "GPIO1_31", "P8_20", 63, -1, -1, "GPMC_CSN2"],
  [ "GPIO1_30", "P8_21", 62, -1, -1, "GPMC_CSN1"],
  [ "GPIO1_5", "P8_22", 37, -1, -1, "GPMC_AD5"],
  [ "GPIO1_4", "P8_23", 36, -1, -1, "GPMC_AD4"],
  [ "GPIO1_1", "P8_24", 33, -1, -1, "GPMC_AD1"],
  [ "GPIO1_0", "P8_25", 32, -1, -1, "GPMC_AD0"],
  [ "GPIO1_29", "P8_26", 61, -1, -1, "GPMC_CSN0"],
  [ "GPIO2_22", "P8_27", 86, -1, -1, "LCD_VSYNC"],
  [ "GPIO2_24", "P8_28", 88, -1, -1, "LCD_PCLK"],
  [ "GPIO2_23", "P8_29", 87, -1, -1, "LCD_HSYNC"],
  [ "GPIO2_25", "P8_30", 89, -1, -1, "LCD_AC_BIAS_EN"],
  [ "UART5_CTSN", "P8_31", 10, -1, -1, "LCD_DATA14"],
  [ "UART5_RTSN", "P8_32", 11, -1, -1, "LCD_DATA15"],
  [ "UART4_RTSN", "P8_33", 9, -1, -1, "LCD_DATA13"],
  [ "UART3_RTSN", "P8_34", 81, 2, -1, "LCD_DATA11"],
  [ "UART4_CTSN", "P8_35", 8, -1, -1, "LCD_DATA12"],
  [ "UART3_CTSN", "P8_36", 80, 2, -1, "LCD_DATA10"],
  [ "UART5_TXD", "P8_37", 78, -1, -1, "LCD_DATA8"],
  [ "UART5_RXD", "P8_38", 79, -1, -1, "LCD_DATA9"],
  [ "GPIO2_12", "P8_39", 76, -1, -1, "LCD_DATA6"],
  [ "GPIO2_13", "P8_40", 77, -1, -1, "LCD_DATA7"],
  [ "GPIO2_10", "P8_41", 74, -1, -1, "LCD_DATA4"],
  [ "GPIO2_11", "P8_42", 75, -1, -1, "LCD_DATA5"],
  [ "GPIO2_8", "P8_43", 72, -1, -1, "LCD_DATA2"],
  [ "GPIO2_9", "P8_44", 73, -1, -1, "LCD_DATA3"],
  [ "GPIO2_6", "P8_45", 70, 3, -1, "LCD_DATA0"],
  [ "GPIO2_7", "P8_46", 71, 3, -1, "LCD_DATA1"],
  [ "DGND", "P9_1", 0, -1, -1, "unused"],
  [ "DGND", "P9_2", 0, -1, -1, "unused"],
  [ "VDD_3V3", "P9_3", 0, -1, -1, "unused"],
  [ "VDD_3V3", "P9_4", 0, -1, -1, "unused"],
  [ "VDD_5V", "P9_5", 0, -1, -1, "unused"],
  [ "VDD_5V", "P9_6", 0, -1, -1, "unused"],
  [ "SYS_5V", "P9_7", 0, -1, -1, "unused"],
  [ "SYS_5V", "P9_8", 0, -1, -1, "unused"],
  [ "PWR_BUT", "P9_9", 0, -1, -1, "unused"],
  [ "SYS_RESETn", "P9_10", 0, -1, -1, "unused"],
  [ "UART4_RXD", "P9_11", 30, -1, -1, "GPMC_WAIT0"],
  [ "GPIO1_28", "P9_12", 60, -1, -1, "GPMC_BEN1"],
  [ "UART4_TXD", "P9_13", 31, -1, -1, "GPMC_WPN"],
  [ "EHRPWM1A", "P9_14", 50, 6, -1, "GPMC_A2"],
  [ "GPIO1_16", "P9_15", 48, -1, -1, "GPMC_A0"],
  [ "EHRPWM1B", "P9_16", 51, 6, -1, "GPMC_A3"],
  [ "I2C1_SCL", "P9_17", 5, -1, -1, "SPI0_CS0"],
  [ "I2C1_SDA", "P9_18", 4, -1, -1, "SPI0_D1"],
  [ "I2C2_SCL", "P9_19", 13, -1, -1, "UART1_RTSN"],
  [ "I2C2_SDA", "P9_20", 12, -1, -1, "UART1_CTSN"],
  [ "UART2_TXD", "P9_21", 3, 3, -1, "SPI0_D0"],
  [ "UART2_RXD", "P9_22", 2, 3, -1, "SPI0_SCLK"],
  [ "GPIO1_17", "P9_23", 49, -1, -1, "GPMC_A1"],
  [ "UART1_TXD", "P9_24", 15, -1, -1, "UART1_TXD"],
  [ "GPIO3_21", "P9_25", 117, -1, -1, "MCASP0_AHCLKX"],
  [ "UART1_RXD", "P9_26", 14, -1, -1, "UART1_RXD"],
  [ "GPIO3_19", "P9_27", 115, -1, -1, "MCASP0_FSR"],
  [ "SPI1_CS0", "P9_28", 113, 4, -1, "MCASP0_AHCLKR"],
  [ "SPI1_D0", "P9_29", 111, 1, -1, "MCASP0_FSX"],
  [ "SPI1_D1", "P9_30", 112, -1, -1, "MCASP0_AXR0"],
  [ "SPI1_SCLK", "P9_31", 110, 1, -1, "MCASP0_ACLKX"],
  [ "VDD_ADC", "P9_32", 0, -1, -1, "unused"],
  [ "AIN4", "P9_33", 0, -1, 4, "unused"],
  [ "GNDA_ADC", "P9_34", 0, -1, -1, "uused"],
  [ "AIN6", "P9_35", 0, -1, 6, "unused"],
  [ "AIN5", "P9_36", 0, -1, 5, "unused"],
  [ "AIN2", "P9_37", 0, -1, 2, "unused"],
  [ "AIN3", "P9_38", 0, -1, 3, "unused"],
  [ "AIN0", "P9_39", 0, -1, 0, "unused"],
  [ "AIN1", "P9_40", 0, -1, 1, "unused"],
  [ "CLKOUT2", "P9_41", 20, -1, -1, "XDMA_EVENT_INTR1"],
  [ "GPIO0_7", "P9_42", 7, 0, -1, "ECAP0_IN_PWM0_OUT"],
  [ "DGND", "P9_43", 0, -1, -1, "unused"],
  [ "DGND", "P9_44", 0, -1, -1, "unused"],
  [ "DGND", "P9_45", 0, -1, -1, "unused"],
  [ "DGND", "P9_46", 0, -1, -1, "unused"],
  
  # Commented out Blue and PocketBeagle since our project doesn't use them
  #  These are for the Blue
  #[ "GP0_3", "GP0_3",  57, -1, -1, "blue"],
  #[ "GP0_4", "GP0_4",  49, -1, -1, "blue"],
  #[ "GP0_5", "GP0_5", 116, -1, -1, "blue"],
  #[ "GP0_6", "GP0_6", 113, -1, -1, "blue"],
  #[ "GP1_3", "GP1_3",  98, -1, -1],
  #[ "GP1_4", "GP1_4",  97, -1, -1],
  #[ "RED_LED",   "RED",    66, -1, -1],   # LEDs
  #[ "GREEN_LED", "GREEN",  67, -1, -1],
  #[ "BAT25", "BAT25",  27, -1, -1],
  #[ "BAT50", "BAT50",  11, -1, -1],
  #[ "BAT75", "BAT75",  61, -1, -1],
  #[ "BAT100", "BAT100",  10000, -1, -1], # Placeholder
  #[ "WIFI", "WIFI",  10001, -1, -1], # Placeholder
  
  #[ "PAUSE", "P8_9",  69, 1, -1],
  #[ "MODE",  "P8_10", 68, 1, -1],
  
  # These are for the PocketBeagle
  #[ "VIN_AC", "P1_1", 0, -1, -1],
  #[ "GPIO2_23", "P1_2", 87, -1, -1],
  #[ "USB1_DRVVBUS", "P1_3", 0, -1, -1],
  #[ "GPIO2_25", "P1_4", 89, -1, -1],
  #[ "USB1_VBUS_IN", "P1_5", 0, -1, -1],
  #[ "SPI0_CS0", "P1_6", 5, -1, -1],
  #[ "VIN-USB", "P1_7", 0, -1, -1],
  #[ "SPI0_SCLK", "P1_8", 2, 3, -1],
  #[ "USB1-DN", "P1_9", 0, -1, -1],
  #[ "SPI0_D0", "P1_10", 3, 3, -1],
  #[ "USB1-DP", "P1_11", 0, -1, -1],
  #[ "SPI0_D1", "P1_12", 4, -1, -1],
  #[ "USB1-ID", "P1_13", 0, -1, -1],
  #[ "VOUT-3.3V", "P1_14", 0, -1, -1],
  #[ "GND", "P1_15", 0, -1, -1],
  #[ "GND", "P1_16", 0, -1, -1],
  #[ "VREFN", "P1_17", 0, -1, -1],
  #[ "VREFP", "P1_18", 0, -1, -1],
  #[ "AIN0", "P1_19", 0, -1, 0],
  #[ "GPIO0_20", "P1_20", 20, 4, -1],
  #[ "AIN1", "P1_21", 0, -1, 1],
  #[ "GND", "P1_22", 0, -1, -1],
  #[ "AIN2", "P1_23", 0, -1, 2],
  #[ "VOUT-5V", "P1_24", 0, -1, -1],
  #[ "AIN3", "P1_25", 0, -1, 3],
  #[ "I2C2_SDA", "P1_26", 12, 1, -1],
  #[ "AIN4", "P1_27", 0, -1, 4],
  #[ "I2C2_SCL", "P1_28", 13, 1, -1],
  #[ "GPIO3_21", "P1_29", 117, -1, -1],
  #[ "UART0_TXD", "P1_30", 43, -1, -1],
  #[ "GPIO3_18", "P1_31", 114, -1, -1],
  #[ "UART0_RXD", "P1_32", 42, -1, -1],
  #[ "GPIO3_15", "P1_33", 111, 1, -1],
  #[ "GPIO0_26", "P1_34", 26, -1, -1],
  #[ "GPIO2_24", "P1_35", 88, -1, -1],
  #[ "EHRPWM0A", "P1_36", 110, 1, -1],
  #[ "EHRPWM1A", "P2_1", 50, 6, -1],
  #[ "GPIO1_27", "P2_2", 59, -1, -1],
  #[ "GPIO0_23", "P2_3", 23, 4, -1],
  #[ "GPIO1_26", "P2_4", 58, -1, -1],
  #[ "UART4_RXD", "P2_5", 30, -1, -1],
  #[ "GPIO1_25", "P2_6", 57, -1, -1],
  #[ "UART4_TXD", "P2_7", 31, -1, -1],
  #[ "GPIO1_28", "P2_8", 60, -1, -1],
  #[ "I2C1_SCL", "P2_9", 15, -1, -1],
  #[ "GPIO1_20", "P2_10", 52, -1, -1],
  #[ "I2C1_SDA", "P2_11", 14, -1, -1],
  #[ "POWER_BUTTON", "P2_12", 0, -1, -1],
  #[ "VOUT-5V", "P2_13", 0, -1, -1],
  #[ "BAT-VIN", "P2_14", 0, -1, -1],
  #[ "GND", "P2_15", 0, -1, -1],
  #[ "BAT-TEMP", "P2_16", 0, -1, -1],
  #[ "GPIO2_1", "P2_17", 65, -1, -1],
  #[ "GPIO1_15", "P2_18", 47, -1, -1],
  #[ "GPIO0_27", "P2_19", 27, -1, -1],
  #[ "GPIO2_0", "P2_20", 64, -1, -1],
  #[ "GND", "P2_21", 0, -1, -1],
  #[ "GPIO1_14", "P2_22", 46, -1, -1],
  #[ "VOUT-3.3V", "P2_23", 0, -1, -1],
  #[ "GPIO1_12", "P2_24", 44, -1, -1],
  #[ "SPI1_CS0", "P2_25", 41, -1, -1],
  #[ "RESET#", "P2_26", 0, -1, -1],
  #[ "SPI1_D0", "P2_27", 40, 5, -1],
  #[ "GPIO3_20", "P2_28", 116, -1, -1],
  #[ "SPI1_SCLK", "P2_29", 7, -1, -1],
  #[ "GPIO3_17", "P2_30", 113, -1, -1],
  #[ "SPI1_CS1", "P2_31", 19, 2, -1],
  #[ "GPIO3_16", "P2_32", 112, -1, -1],
  #[ "GPIO1_13", "P2_33", 45, -1, -1],
  #[ "GPIO3_19", "P2_34", 115, -1, -1],
  #[ "GPIO2_22", "P2_35", 86, -1, -1],
  #[ "AIN7", "P2_36", 0, -1, 7],
  
  [ None, None, 0, 0, 0, "unused" ]
]
def channelNameConvert(channel):
    global gChannel
    global gChannelNames
    for i in range(0, len(gChannel)):
        if channel == gChannelNames[i]:
            return gChannel[i]
    print("Error: name not found")
    return "error"

def run(channel, edge, callback=None, debounce=0, child_conn=None):
    channel = channelNameConvert(channel)
    thread_go = True
    while thread_go:
        event_detected = None
        thread_go = wait_for_edge(channel, edge, child_conn)
        if thread_go:
            time.sleep(debounce/1000000.0)
            callback(channel)
        #thread_go = not child_conn.poll()
        #if not thread_go:
        #    child_conn.close()

def setup(channel, direction):
    """Set up the GPIO channel, direction and (optional) pull/up down control.
    
    channel        - channel can be in the form of 'P8_10', or 'EHRPWM2A'
    direction      - INPUT or OUTPUT
    [pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN
    [initial]      - Initial value for an output channel
    [delay]        - Time in milliseconds to wait after exporting gpio pin"""
    global gChannel
    global gChannelNames
    for index in table:
       if index[1] == channel:
           channel = index[5]
           gChannel.append(index[5])
           gChannelNames.append(index[1])
           break
       elif index[0] == channel:
           channel = index[5]
           gChannel.append(index[5])
           gChannelNames.append(index[0])
           break
       elif index[5] == channel:
           gChannel.append(index[5])
           gChannelNames.append(index[5])
    found=0
# Searh for channel in either name or consumer
    for chip in gpiod.ChipIter():
        # print('[] - [] lines:'.format(chip.name(), chip.num_lines()))

        for line in gpiod.LineIter(chip):
            offset = line.offset()
            name = line.name()
            consumer = line.consumer()
            linedirection = line.direction()
            active_state = line.active_state()
            
            if name == channel or consumer == channel:

                # print('[]\tline [:>3]: [:>18] [:>12] [:>8] [:>10]'.format(
                #         chip.name(),
                #         offset,
                #         'unnamed' if name is None else name,
                #         'unused' if consumer is None else consumer,
                #         'input' if linedirection == gpiod.Line.DIRECTION_INPUT else 'output',
                #         'active-low' if active_state == gpiod.Line.ACTIVE_LOW else 'active-high'))
                found=1
                break
        if found:
            break

        chip.close()

    if not found: 
        print(channel + ': Not found')
        sys.exit(1)
    
    # print(chip)
    
    lines = chip.get_lines([offset])
    # print(lines)
    if direction == IN:
        ret = lines.request(consumer=CONSUMER, type=gpiod.LINE_REQ_DIR_IN)
    elif direction == OUT:
        ret = lines.request(consumer=CONSUMER, type=gpiod.LINE_REQ_DIR_OUT)
    else:
        print("Unknown direction: " + str(direction))
        sys.exit(1)
    
    if ret:
        print(ret)
        
    ports[channel] = [lines, chip]
    # print(ports)

def output(channel, vals):
    """Output to a GPIO channel
    
    channel  - gpio channel
    value - 0/1 or False/True or LOW/HIGH"""
    # print("output()")
    print(channel)
    channel = channelNameConvert(channel)
    print(channel)
    if type(vals) is not type([]):
       vals = [vals]
    ret = ports[channel][0].set_values(vals)
    if ret:
        print(ret)

def input(channel):
    """Input from a GPIO channel.  Returns HIGH=1=True or LOW=0=False
    gpio - gpio channel"""
    # print("input()")
    # print(channel)
    channel = channelNameConvert(channel)
    return ports[channel][0].get_values()

def wait_for_edge(channel, edge, child_conn, timeout = -1):
    """Wait for an edge.
    
    channel - gpio channel
    edge - RISING, FALLING or BOTH
    timeout (optional) - time to wait in miliseconds. -1 will wait forever (default)"""
    # print("wait_for_edge()")
    # print(ports)
    channel = channelNameConvert(channel)
    line=ports[channel][0]
    chip=ports[channel][1]
    
    if edge == RISING:
        ev_edge = gpiod.LINE_REQ_EV_RISING_EDGE
    elif edge == FALLING:
        ev_edge = gpiod.LINE_REQ_EV_FALLING_EDGE
    elif edge == BOTH:
        ev_edge = gpiod.LINE_REQ_EV_BOTH_EDGES
    else:
        print("Unknown edge type: " + str(edge))
    
    # Try releasing the line and requesting again
    
    offset = line.to_list()[0].offset()
    line.release()
    line = chip.get_lines([offset])
    
    line.request(consumer=CONSUMER, type=ev_edge)
    x = None
    while not x:
        thread_go = not child_conn.poll()
        if not thread_go:
            return False 
        x = line.event_wait(sec = 1)
    return True

def add_event_detect(channel, edge, callback=None, debounce=0):
    """Enable edge detection events for a particular GPIO channel.
  
    channel      - board pin number.
    edge         - RISING, FALLING or BOTH
    [callback]   - A callback function for the event (optional)
    [bouncetime] - Switch bounce timeout in ms for callback"""
    global threads
    global parent_conn
    channel = channelNameConvert(channel)
    parent_conn_temp, child_conn  = Pipe()
    parent_conn.append(parent_conn_temp)
    process2 = Process(target = run, args = (channel, edge, callback, debounce, child_conn))
    threads.append(process2)
    process2.start()
    return
       
def cleanup():
    """Clean up by resetting all GPIO channels that have been used by 
    this program to INPUT with no pullup/pulldown and no event detection."""
    global ports
    global threads
    i = 0
    for thread in threads:
        parent_conn[i].send(False)
        parent_conn[i].close()
        thread.join()
        thread.close()
        i = i + 1
    for channel, val in ports.items():
        ret = val[0].release()
        if ret:
            print(ret)
        ret = val[1].close()
        if ret:
            print(ret)
        ports={}



#!/usr/bin/env python3
"""PWM functionality of a BeagleBone using Python."""
import glob
import time

t_NULL=0
t_module = 0
t_sysfs  = 1
t_index  = 2
t_muxmode= 3
t_path   = 4
t_name   = 5
t_chip   = 6
t_addr   = 7
t_key    = 8

pwm_table = [
  [ "ehrpwm2", 6, 1, 4, "ehrpwm.2:1", "EHRPWM2B", "48304000", "48304200", "P8_13"],
  [ "ehrpwm2", 5, 0, 4, "ehrpwm.2:0", "EHRPWM2A", "48304000", "48304200", "P8_19"],
  [ "ehrpwm1", 4, 1, 2, "ehrpwm.1:1", "EHRPWM1B", "48302000", "48302200", "P8_34"],
  [ "ehrpwm1", 3, 0, 2, "ehrpwm.1:0", "EHRPWM1A", "48302000", "48302200", "P8_36"],
  [ "ehrpwm2", 5, 0, 3, "ehrpwm.2:0", "EHRPWM2A", "48304000", "48304200", "P8_45"],
  [ "ehrpwm2", 6, 1, 3, "ehrpwm.2:1", "EHRPWM2B", "48304000", "48304200", "P8_46"],
  [ "ehrpwm1", 3, 0, 6, "ehrpwm.1:0", "EHRPWM1A", "48302000", "48302200", "P9_14"],
  [ "ehrpwm1", 4, 1, 6, "ehrpwm.1:1", "EHRPWM1B", "48302000", "48302200", "P9_16"],
  [ "ehrpwm0", 1, 1, 3, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P9_21"],
  [ "ehrpwm0", 0, 0, 3, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P9_22"],
  [   "ecap2", 7, 0, 4, "ecap.2",     "ECAPPWM2", "48304000", "48304100", "P9_28"],
  [ "ehrpwm0", 1, 1, 1, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P9_29"],
  [ "ehrpwm0", 0, 0, 1, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P9_31"],
  [   "ecap0", 2, 0, 0, "ecap.0",     "ECAPPWM0", "48300000", "48300100", "P9_42"],
  [ "timer4", 0, 0, 2, "", "", "", "dmtimer-pwm-4", "P8_7" ],
  [ "timer7", 0, 0, 2, "", "", "", "dmtimer-pwm-7", "P8_8" ],
  [ "timer5", 0, 0, 2, "", "", "", "dmtimer-pwm-5", "P8_9" ],
  [ "timer6", 0, 0, 2, "", "", "", "dmtimer-pwm-6", "P8_10" ],
  [ "ehrpwm0", 0, 0, 1, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P1_8"],
  [ "ehrpwm0", 0, 0, 1, "ehrpwm.0:0", "EHRPWM0A", "48300000", "48300200", "P1_36"],
  [ "ehrpwm0", 1, 1, 1, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P1_10"],
  [ "ehrpwm0", 1, 1, 1, "ehrpwm.0:1", "EHRPWM0B", "48300000", "48300200", "P1_33"],
  [ "ehrpwm1", 3, 0, 6, "ehrpwm.1:0", "EHRPWM1A", "48302000", "48302200", "P2_1"],
  [ "ehrpwm2", 6, 1, 3, "ehrpwm.2:1", "EHRPWM2B", "48304000", "48304200", "P2_3"],
  [ "timer7", 0, 0, 4, "", "", "", "dmtimer-pwm-7", "P1_20" ],
  [ "timer6", 0, 0, 1, "", "", "", "dmtimer-pwm-6", "P1_26" ],
  [ "timer5", 0, 0, 1, "", "", "", "dmtimer-pwm-5", "P1_28" ],
  [ "timer7", 0, 0, 5, "", "", "", "dmtimer-pwm-7", "P2_27" ],
  [ "timer4", 0, 0, 2, "", "", "", "dmtimer-pwm-4", "P2_31" ],
]

# This is needed to access the timer pwm's
# uboot_overlay_addr0=/lib/firmware/BB-PWM-TIMER-P8.07.dtbo
# uboot_overlay_addr1=/lib/firmware/BB-PWM-TIMER-P8.08.dtbo
# uboot_overlay_addr2=/lib/firmware/BB-PWM-TIMER-P8.09.dtbo
# uboot_overlay_addr3=/lib/firmware/BB-PWM-TIMER-P8.10.dtbo
# https://github.com/adafruit/adafruit-beaglebone-io-python/issues/229

# print(pwm_table[2][t_key])
# print(len(pwm_table))

def get_pwm_key(channel):
    for i in range(len(pwm_table)):
        if pwm_table[i][t_key] == channel:
            return pwm_table[i]
    print(channel + ': Not Found')
    return None

def get_pwm_path(channel):
    x = get_pwm_key(channel)
    if x == None:
        return None
    pwmPath =  glob.glob("/sys/devices/platform/ocp/" + x[t_chip] + ".epwmss/" 
            + x[t_addr] + ".pwm/pwm/*")[0]
    # print(pwmPath)
    return [pwmPath, x[t_index]]

def start(channel, duty, freq=2000, polarity=0):
    """Set up and start the PWM channel.
    
    channel can be in the form of 'P8_10', or 'EHRPWM2A'"""
    print(channel)
    path = get_pwm_path(channel)
    if path == None:
        return None
    # /sys/devices/platform/ocp/48302000.epwmss/48302200.pwm/pwm/pwmchip4/pwm-4:0
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])
    # print(pathpwm)

    period_ns = 1e9 / freq
    duty_ns = period_ns * (duty / 100.0)

    # export
    try:
        fd = open(path[0] + "/export", 'w')
        fd.write(str(path[1])) 
        fd.close()
    except:
        pass
    time.sleep(0.05)    # Give export a chance
    
    set_pin_mode(channel, 'pwm')
    
    # Duty Cycle - Set to 0 so period can be changed
    try:
        fd = open(pathpwm + "/duty_cycle", 'w')
        fd.write('0')
        fd.close()
    except:
        pass

    # Period
    fd = open(pathpwm + "/period", 'w')
    fd.write(str(int(period_ns)))
    fd.close()

    # Duty Cycle
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write(str(int(duty_ns)))
    fd.close()

    # Enable
    fd = open(pathpwm + "/enable", 'w')
    fd.write('1')
    fd.close()
    
    return 'Started'

def set_frequency(channel, freq):
    """Change the frequency
    
    frequency - frequency in Hz (freq > 0.0)"""
    path = get_pwm_path(channel)
    # /sys/devices/platform/ocp/48302000.epwmss/48302200.pwm/pwm/pwmchip4/pwm-4:0
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])

    # compute current duty cycle
    fd = open(pathpwm + "/duty_cycle", 'r')
    duty_cycle_ns = int(fd.read()[:-1] )       # Remove \n at end
    fd.close()
    
    fd = open(pathpwm + "/period", 'r')
    period_ns = int(fd.read()[:-1])
    fd.close()
    
    duty_cycle = duty_cycle_ns/period_ns          # compute current duty cycle as fraction
    period_ns = 1e9 / freq                  # compute new period
    duty_cycle_ns = period_ns*duty_cycle # compute new duty cycle as fraction of period

    # print('duty_cycle: ' + str(duty_cycle))
    # print('period_ns: ' + str(period_ns))
    # print('duty_cycle_ns: ' + str(duty_cycle_ns))

    # Duty Cycle - Set to 0
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write('0')
    fd.close()
    
    # Period
    fd = open(pathpwm + "/period", 'w')
    fd.write(str(int(period_ns)))
    fd.close()

    # Duty Cycle
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write(str(int(duty_cycle_ns)))
    fd.close()

def set_duty_cycle(channel, duty):
    """Change the duty cycle.
    
    dutycycle - between 0.0 and 100.0"""
    path = get_pwm_path(channel)
    # /sys/devices/platform/ocp/48302000.epwmss/48302200.pwm/pwm/pwmchip4/pwm-4:0
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])

    fd = open(pathpwm + "/period", 'r')
    period_ns = int(fd.read()[:-1])
    fd.close()

    duty_cycle_ns = duty/100 * period_ns
    # Duty Cycle
    fd = open(pathpwm + "/duty_cycle", 'w')
    fd.write(str(int(duty_cycle_ns)))
    fd.close()
    
def stop(channel):
    """Stop the PWM channel.
    
    channel can be in the form of 'P8_10', or 'EHRPWM2A'"""
    path = get_pwm_path(channel)
    pathpwm = path[0] + "/pwm-" + path[0][-1] + ':' + str(path[1])
   
    # Disable
    fd = open(pathpwm + "/enable", 'w')
    fd.write('0')
    fd.close()

    # unexport
    fd = open(path[0] + "/unexport", 'w')
    fd.write(str(path[1])) 
    fd.close()
    
    set_pin_mode(channel, 'gpio')

def set_pin_mode(channel, mode):
    path = '/sys/devices/platform/ocp/ocp:' + channel + '_pinmux/state'
    fd = open(path, 'w')
    fd.write(mode)
    fd.close()

    return path



# import importlib

# common = input('common')
# importlib.import_module(common)
import os

uart_table = [
  [ "UART1", "/dev/ttyO1", "ADAFRUIT-UART1", "P9_26", "P9_24"],
  [ "UART2", "/dev/ttyO2", "ADAFRUIT-UART2", "P9_22", "P9_21"],
  [ "UART3", "/dev/ttyO3", "ADAFRUIT-UART3", "P9_42", ""],
  [ "UART4", "/dev/ttyO4", "ADAFRUIT-UART4", "P9_11", "P9_13"],
  [ "UART5", "/dev/ttyO5", "ADAFRUIT-UART5", "P8_38", "P8_37"],
  [ "PB-UART0", "/dev/ttyO0", "ADAFRUIT-UART0", "P1_30", "P1_32"],
  [ "PB-UART1", "/dev/ttyO1", "ADAFRUIT-UART1", "P2_11", "P2_09"],
  [ "PB-UART2", "/dev/ttyO2", "ADAFRUIT-UART2", "P1_08", "P1_10"],
  [ "PB-UART4", "/dev/ttyO4", "ADAFRUIT-UART4", "P2_05", "P2_07"],
  [ None, None, 0, 0, 0 ]
]

# uboot overlay
"""def uboot_overlay_enabled():
	cmd = "/bin/grep -c bone_capemgr.uboot_capemgr_enabled=1 /proc/cmdline"
	
	file = os.popen(cmd, mode="r")
	if file == None:
			print("error: uboot_overlay_enabled() failed to run cmd = {}".format(cmd))
			return -1
	uboot_overlay = file.read(1)
	file.close()
	
	if uboot_overlay == '1':
		print("Adafruit_BBIO: uboot_overlay_enabled() is true")
		return 1
	else:
		print("Adafruit_BBIO: uboot_overlay_enabled() is false")
		return 0
"""

	
def uart_setup(name):
	for k in range(0,5):
		if name == uart_table[k][2]:
			if k == 2:
				file_dir= "/sys/devices/platform/ocp/ocp:" + uart_table[2][3] + "_pinmux/state"
				state = open(file_dir, mode='w')
				state.write("uart")
				state.close()
			else:
				for i in range(0, 2):
					file_dir = "/sys/devices/platform/ocp/ocp:" + uart_table[k][3+i] + "_pinmux/state"
					state = open(file_dir, mode='w')
					state.write("uart")
					state.close()

def uart_cleanup():
	for k in range(0,5):
		if k == 2:
			file_dir = "/sys/devices/platform/ocp/ocp:" + uart_table[2][3] + "_pinmux/state"
			state = open(file_dir, mode='w')
			state.write("default")
			state.close()
		else:
			for i in range(0, 2):
				file_dir = "/sys/devices/platform/ocp/ocp:" + uart_table[k][3+i] + "_pinmux/state"
				state = open(file_dir, mode='w')
				state.write("default")
				state.close()
	# e1 = unload_device_tree("ADAFRUIT-UART1")
	# e2 = unload_device_tree("ADAFRUIT-UART2")
	# e3 = unload_device_tree("ADAFRUIT-UART3")
	# e4 = unload_device_tree("ADAFRUIT-UART4")
	# e5 = unload_device_tree("ADAFRUIT-UART5")
	# if e1 is not "OK":
	#	return e1
	# if e2 is not "OK":
	#	return e2
	# if e3 is not "OK":
	#	return e3
	# if e4 is not "OK":
	#	return e4
	#if e5 is not "OK":
	#	return e5

	#return "OK"




import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="BBIOPY",
	version="0.1.5",
	author="Eric Morse, Joshua Key, and Mark Yoder",
	author_email="morsee@rose-hulman.edu",
	description="Full python implementation of BBIO gpiod",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/EricMorse/ECE434-Project",
	packages=setuptools.find_packages(),
	classifiers=[
		"Programming Language :: Python :: 3",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.6',
)



#!/usr/bin/env python3
"""gpiod-based GPIO functionality of a BeagleBone using Python."""
import gpiod
import sys
#import multiprocessing
from multiprocessing import Pipe, Process
# import select
import time

ALT0 = 4
BOTH = 3
FALLING = 2
HIGH = 1
IN = 0
LOW = 0
OUT = 1
PUD_DOWN = 1
PUD_OFF = 0
PUD_UP = 2
RISING = 1
VERSION = '0.0.0'
threads=[]
ports={}        # Dictionary of channel/line pairs that are open
CONSUMER='GPIOmay'
parent_conn = []
gChannel = []
gChannelNames = []

# Table generated based on https://github.com/jadonk/bonescript/blob/master/src/bone.js
table = [
  [ "USR0", "USR0", 53, -1, -1, "GPMC_A5"],
  [ "USR1", "USR1", 54, -1, -1, "GPMC_A6"],
  [ "USR2", "USR2", 55, -1, -1, "GPMC_A7"],
  [ "USR3", "USR3", 56, -1, -1, "GPMC_A8"],
  [ "DGND", "P8_1", 0, -1, -1, "unused"],
  [ "DGND", "P8_2", 0, -1, -1, "unused"],
  [ "GPIO1_6", "P8_3", 38, -1, -1, "GPMC_AD6"],
  [ "GPIO1_7", "P8_4", 39, -1, -1, "GPMC_AD7"],
  [ "GPIO1_2", "P8_5", 34, -1, -1, "GPMC_AD2"],
  [ "GPIO1_3", "P8_6", 35, -1, -1, "GPMC_AD3"],
  [ "TIMER4", "P8_7", 66, 2, -1, "GPMC_ADVN_ALE"],
  [ "TIMER7", "P8_8", 67, 2, -1, "GPMC_OEN_REN"],
  [ "TIMER5", "P8_9", 69, 2, -1, "GPMC_BEON_CLE"],
  [ "TIMER6", "P8_10", 68, 2, -1, "GPMC_WEN"],
  [ "GPIO1_13", "P8_11", 45, -1, -1, "GPMC_AD13"],
  [ "GPIO1_12", "P8_12", 44, -1, -1, "GPMC_AD12"],
  [ "EHRPWM2B", "P8_13", 23, 4, -1, "GPMC_AD9"],
  [ "GPIO0_26", "P8_14", 26, -1, -1, "GPMC_AD10"],
  [ "GPIO1_15", "P8_15", 47, -1, -1, "GPMC_AD15"],
  [ "GPIO1_14", "P8_16", 46, -1, -1, "GPMC_AD14"],
  [ "GPIO0_27", "P8_17", 27, -1, -1, "GPMC_AD11"],
  [ "GPIO2_1", "P8_18", 65, -1, -1, "GPMC_DK_MUX0"],
  [ "EHRPWM2A", "P8_19", 22, 4, -1, "GPMC_AD8"],
  [ "GPIO1_31", "P8_20", 63, -1, -1, "GPMC_CSN2"],
  [ "GPIO1_30", "P8_21", 62, -1, -1, "GPMC_CSN1"],
  [ "GPIO1_5", "P8_22", 37, -1, -1, "GPMC_AD5"],
  [ "GPIO1_4", "P8_23", 36, -1, -1, "GPMC_AD4"],
  [ "GPIO1_1", "P8_24", 33, -1, -1, "GPMC_AD1"],
  [ "GPIO1_0", "P8_25", 32, -1, -1, "GPMC_AD0"],
  [ "GPIO1_29", "P8_26", 61, -1, -1, "GPMC_CSN0"],
  [ "GPIO2_22", "P8_27", 86, -1, -1, "LCD_VSYNC"],
  [ "GPIO2_24", "P8_28", 88, -1, -1, "LCD_PCLK"],
  [ "GPIO2_23", "P8_29", 87, -1, -1, "LCD_HSYNC"],
  [ "GPIO2_25", "P8_30", 89, -1, -1, "LCD_AC_BIAS_EN"],
  [ "UART5_CTSN", "P8_31", 10, -1, -1, "LCD_DATA14"],
  [ "UART5_RTSN", "P8_32", 11, -1, -1, "LCD_DATA15"],
  [ "UART4_RTSN", "P8_33", 9, -1, -1, "LCD_DATA13"],
  [ "UART3_RTSN", "P8_34", 81, 2, -1, "LCD_DATA11"],
  [ "UART4_CTSN", "P8_35", 8, -1, -1, "LCD_DATA12"],
  [ "UART3_CTSN", "P8_36", 80, 2, -1, "LCD_DATA10"],
  [ "UART5_TXD", "P8_37", 78, -1, -1, "LCD_DATA8"],
  [ "UART5_RXD", "P8_38", 79, -1, -1, "LCD_DATA9"],
  [ "GPIO2_12", "P8_39", 76, -1, -1, "LCD_DATA6"],
  [ "GPIO2_13", "P8_40", 77, -1, -1, "LCD_DATA7"],
  [ "GPIO2_10", "P8_41", 74, -1, -1, "LCD_DATA4"],
  [ "GPIO2_11", "P8_42", 75, -1, -1, "LCD_DATA5"],
  [ "GPIO2_8", "P8_43", 72, -1, -1, "LCD_DATA2"],
  [ "GPIO2_9", "P8_44", 73, -1, -1, "LCD_DATA3"],
  [ "GPIO2_6", "P8_45", 70, 3, -1, "LCD_DATA0"],
  [ "GPIO2_7", "P8_46", 71, 3, -1, "LCD_DATA1"],
  [ "DGND", "P9_1", 0, -1, -1, "unused"],
  [ "DGND", "P9_2", 0, -1, -1, "unused"],
  [ "VDD_3V3", "P9_3", 0, -1, -1, "unused"],
  [ "VDD_3V3", "P9_4", 0, -1, -1, "unused"],
  [ "VDD_5V", "P9_5", 0, -1, -1, "unused"],
  [ "VDD_5V", "P9_6", 0, -1, -1, "unused"],
  [ "SYS_5V", "P9_7", 0, -1, -1, "unused"],
  [ "SYS_5V", "P9_8", 0, -1, -1, "unused"],
  [ "PWR_BUT", "P9_9", 0, -1, -1, "unused"],
  [ "SYS_RESETn", "P9_10", 0, -1, -1, "unused"],
  [ "UART4_RXD", "P9_11", 30, -1, -1, "GPMC_WAIT0"],
  [ "GPIO1_28", "P9_12", 60, -1, -1, "GPMC_BEN1"],
  [ "UART4_TXD", "P9_13", 31, -1, -1, "GPMC_WPN"],
  [ "EHRPWM1A", "P9_14", 50, 6, -1, "GPMC_A2"],
  [ "GPIO1_16", "P9_15", 48, -1, -1, "GPMC_A0"],
  [ "EHRPWM1B", "P9_16", 51, 6, -1, "GPMC_A3"],
  [ "I2C1_SCL", "P9_17", 5, -1, -1, "SPI0_CS0"],
  [ "I2C1_SDA", "P9_18", 4, -1, -1, "SPI0_D1"],
  [ "I2C2_SCL", "P9_19", 13, -1, -1, "UART1_RTSN"],
  [ "I2C2_SDA", "P9_20", 12, -1, -1, "UART1_CTSN"],
  [ "UART2_TXD", "P9_21", 3, 3, -1, "SPI0_D0"],
  [ "UART2_RXD", "P9_22", 2, 3, -1, "SPI0_SCLK"],
  [ "GPIO1_17", "P9_23", 49, -1, -1, "GPMC_A1"],
  [ "UART1_TXD", "P9_24", 15, -1, -1, "UART1_TXD"],
  [ "GPIO3_21", "P9_25", 117, -1, -1, "MCASP0_AHCLKX"],
  [ "UART1_RXD", "P9_26", 14, -1, -1, "UART1_RXD"],
  [ "GPIO3_19", "P9_27", 115, -1, -1, "MCASP0_FSR"],
  [ "SPI1_CS0", "P9_28", 113, 4, -1, "MCASP0_AHCLKR"],
  [ "SPI1_D0", "P9_29", 111, 1, -1, "MCASP0_FSX"],
  [ "SPI1_D1", "P9_30", 112, -1, -1, "MCASP0_AXR0"],
  [ "SPI1_SCLK", "P9_31", 110, 1, -1, "MCASP0_ACLKX"],
  [ "VDD_ADC", "P9_32", 0, -1, -1, "unused"],
  [ "AIN4", "P9_33", 0, -1, 4, "unused"],
  [ "GNDA_ADC", "P9_34", 0, -1, -1, "uused"],
  [ "AIN6", "P9_35", 0, -1, 6, "unused"],
  [ "AIN5", "P9_36", 0, -1, 5, "unused"],
  [ "AIN2", "P9_37", 0, -1, 2, "unused"],
  [ "AIN3", "P9_38", 0, -1, 3, "unused"],
  [ "AIN0", "P9_39", 0, -1, 0, "unused"],
  [ "AIN1", "P9_40", 0, -1, 1, "unused"],
  [ "CLKOUT2", "P9_41", 20, -1, -1, "XDMA_EVENT_INTR1"],
  [ "GPIO0_7", "P9_42", 7, 0, -1, "ECAP0_IN_PWM0_OUT"],
  [ "DGND", "P9_43", 0, -1, -1, "unused"],
  [ "DGND", "P9_44", 0, -1, -1, "unused"],
  [ "DGND", "P9_45", 0, -1, -1, "unused"],
  [ "DGND", "P9_46", 0, -1, -1, "unused"],
  
  # Commented out Blue and PocketBeagle since our project doesn't use them
  #  These are for the Blue
  #[ "GP0_3", "GP0_3",  57, -1, -1, "blue"],
  #[ "GP0_4", "GP0_4",  49, -1, -1, "blue"],
  #[ "GP0_5", "GP0_5", 116, -1, -1, "blue"],
  #[ "GP0_6", "GP0_6", 113, -1, -1, "blue"],
  #[ "GP1_3", "GP1_3",  98, -1, -1],
  #[ "GP1_4", "GP1_4",  97, -1, -1],
  #[ "RED_LED",   "RED",    66, -1, -1],   # LEDs
  #[ "GREEN_LED", "GREEN",  67, -1, -1],
  #[ "BAT25", "BAT25",  27, -1, -1],
  #[ "BAT50", "BAT50",  11, -1, -1],
  #[ "BAT75", "BAT75",  61, -1, -1],
  #[ "BAT100", "BAT100",  10000, -1, -1], # Placeholder
  #[ "WIFI", "WIFI",  10001, -1, -1], # Placeholder
  
  #[ "PAUSE", "P8_9",  69, 1, -1],
  #[ "MODE",  "P8_10", 68, 1, -1],
  
  # These are for the PocketBeagle
  #[ "VIN_AC", "P1_1", 0, -1, -1],
  #[ "GPIO2_23", "P1_2", 87, -1, -1],
  #[ "USB1_DRVVBUS", "P1_3", 0, -1, -1],
  #[ "GPIO2_25", "P1_4", 89, -1, -1],
  #[ "USB1_VBUS_IN", "P1_5", 0, -1, -1],
  #[ "SPI0_CS0", "P1_6", 5, -1, -1],
  #[ "VIN-USB", "P1_7", 0, -1, -1],
  #[ "SPI0_SCLK", "P1_8", 2, 3, -1],
  #[ "USB1-DN", "P1_9", 0, -1, -1],
  #[ "SPI0_D0", "P1_10", 3, 3, -1],
  #[ "USB1-DP", "P1_11", 0, -1, -1],
  #[ "SPI0_D1", "P1_12", 4, -1, -1],
  #[ "USB1-ID", "P1_13", 0, -1, -1],
  #[ "VOUT-3.3V", "P1_14", 0, -1, -1],
  #[ "GND", "P1_15", 0, -1, -1],
  #[ "GND", "P1_16", 0, -1, -1],
  #[ "VREFN", "P1_17", 0, -1, -1],
  #[ "VREFP", "P1_18", 0, -1, -1],
  #[ "AIN0", "P1_19", 0, -1, 0],
  #[ "GPIO0_20", "P1_20", 20, 4, -1],
  #[ "AIN1", "P1_21", 0, -1, 1],
  #[ "GND", "P1_22", 0, -1, -1],
  #[ "AIN2", "P1_23", 0, -1, 2],
  #[ "VOUT-5V", "P1_24", 0, -1, -1],
  #[ "AIN3", "P1_25", 0, -1, 3],
  #[ "I2C2_SDA", "P1_26", 12, 1, -1],
  #[ "AIN4", "P1_27", 0, -1, 4],
  #[ "I2C2_SCL", "P1_28", 13, 1, -1],
  #[ "GPIO3_21", "P1_29", 117, -1, -1],
  #[ "UART0_TXD", "P1_30", 43, -1, -1],
  #[ "GPIO3_18", "P1_31", 114, -1, -1],
  #[ "UART0_RXD", "P1_32", 42, -1, -1],
  #[ "GPIO3_15", "P1_33", 111, 1, -1],
  #[ "GPIO0_26", "P1_34", 26, -1, -1],
  #[ "GPIO2_24", "P1_35", 88, -1, -1],
  #[ "EHRPWM0A", "P1_36", 110, 1, -1],
  #[ "EHRPWM1A", "P2_1", 50, 6, -1],
  #[ "GPIO1_27", "P2_2", 59, -1, -1],
  #[ "GPIO0_23", "P2_3", 23, 4, -1],
  #[ "GPIO1_26", "P2_4", 58, -1, -1],
  #[ "UART4_RXD", "P2_5", 30, -1, -1],
  #[ "GPIO1_25", "P2_6", 57, -1, -1],
  #[ "UART4_TXD", "P2_7", 31, -1, -1],
  #[ "GPIO1_28", "P2_8", 60, -1, -1],
  #[ "I2C1_SCL", "P2_9", 15, -1, -1],
  #[ "GPIO1_20", "P2_10", 52, -1, -1],
  #[ "I2C1_SDA", "P2_11", 14, -1, -1],
  #[ "POWER_BUTTON", "P2_12", 0, -1, -1],
  #[ "VOUT-5V", "P2_13", 0, -1, -1],
  #[ "BAT-VIN", "P2_14", 0, -1, -1],
  #[ "GND", "P2_15", 0, -1, -1],
  #[ "BAT-TEMP", "P2_16", 0, -1, -1],
  #[ "GPIO2_1", "P2_17", 65, -1, -1],
  #[ "GPIO1_15", "P2_18", 47, -1, -1],
  #[ "GPIO0_27", "P2_19", 27, -1, -1],
  #[ "GPIO2_0", "P2_20", 64, -1, -1],
  #[ "GND", "P2_21", 0, -1, -1],
  #[ "GPIO1_14", "P2_22", 46, -1, -1],
  #[ "VOUT-3.3V", "P2_23", 0, -1, -1],
  #[ "GPIO1_12", "P2_24", 44, -1, -1],
  #[ "SPI1_CS0", "P2_25", 41, -1, -1],
  #[ "RESET#", "P2_26", 0, -1, -1],
  #[ "SPI1_D0", "P2_27", 40, 5, -1],
  #[ "GPIO3_20", "P2_28", 116, -1, -1],
  #[ "SPI1_SCLK", "P2_29", 7, -1, -1],
  #[ "GPIO3_17", "P2_30", 113, -1, -1],
  #[ "SPI1_CS1", "P2_31", 19, 2, -1],
  #[ "GPIO3_16", "P2_32", 112, -1, -1],
  #[ "GPIO1_13", "P2_33", 45, -1, -1],
  #[ "GPIO3_19", "P2_34", 115, -1, -1],
  #[ "GPIO2_22", "P2_35", 86, -1, -1],
  #[ "AIN7", "P2_36", 0, -1, 7],
  
  [ None, None, 0, 0, 0, "unused" ]
]
def channelNameConvert(channel):
    global gChannel
    global gChannelNames
    for i in range(0, len(gChannel)):
        if channel == gChannelNames[i]:
            return gChannel[i]
    print("Error: name not found")
    return "error"

def run(channel, edge, callback=None, debounce=0, child_conn=None):
    channel = channelNameConvert(channel)
    thread_go = True
    while thread_go:
        event_detected = None
        thread_go = wait_for_edge(channel, edge, child_conn)
        if thread_go:
            time.sleep(debounce/1000000.0)
            callback(channel)
        #thread_go = not child_conn.poll()
        #if not thread_go:
        #    child_conn.close()

def setup(channel, direction):
    """Set up the GPIO channel, direction and (optional) pull/up down control.
    
    channel        - channel can be in the form of 'P8_10', or 'EHRPWM2A'
    direction      - INPUT or OUTPUT
    [pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN
    [initial]      - Initial value for an output channel
    [delay]        - Time in milliseconds to wait after exporting gpio pin"""
    global gChannel
    global gChannelNames
    for index in table:
       if index[1] == channel:
           channel = index[5]
           gChannel.append(index[5])
           gChannelNames.append(index[1])
           break
       elif index[0] == channel:
           channel = index[5]
           gChannel.append(index[5])
           gChannelNames.append(index[0])
           break
       elif index[5] == channel:
           gChannel.append(index[5])
           gChannelNames.append(index[5])
    found=0
# Searh for channel in either name or consumer
    for chip in gpiod.ChipIter():
        # print('[] - [] lines:'.format(chip.name(), chip.num_lines()))

        for line in gpiod.LineIter(chip):
            offset = line.offset()
            name = line.name()
            consumer = line.consumer()
            linedirection = line.direction()
            active_state = line.active_state()
            
            if name == channel or consumer == channel:

                # print('[]\tline [:>3]: [:>18] [:>12] [:>8] [:>10]'.format(
                #         chip.name(),
                #         offset,
                #         'unnamed' if name is None else name,
                #         'unused' if consumer is None else consumer,
                #         'input' if linedirection == gpiod.Line.DIRECTION_INPUT else 'output',
                #         'active-low' if active_state == gpiod.Line.ACTIVE_LOW else 'active-high'))
                found=1
                break
        if found:
            break

        chip.close()

    if not found: 
        print(channel + ': Not found')
        sys.exit(1)
    
    # print(chip)
    
    lines = chip.get_lines([offset])
    # print(lines)
    if direction == IN:
        ret = lines.request(consumer=CONSUMER, type=gpiod.LINE_REQ_DIR_IN)
    elif direction == OUT:
        ret = lines.request(consumer=CONSUMER, type=gpiod.LINE_REQ_DIR_OUT)
    else:
        print("Unknown direction: " + str(direction))
        sys.exit(1)
    
    if ret:
        print(ret)
        
    ports[channel] = [lines, chip]
    # print(ports)

def output(channel, vals):
    """Output to a GPIO channel
    
    channel  - gpio channel
    value - 0/1 or False/True or LOW/HIGH"""
    # print("output()")
    print(channel)
    channel = channelNameConvert(channel)
    print(channel)
    if type(vals) is not type([]):
       vals = [vals]
    ret = ports[channel][0].set_values(vals)
    if ret:
        print(ret)

def input(channel):
    """Input from a GPIO channel.  Returns HIGH=1=True or LOW=0=False
    gpio - gpio channel"""
    # print("input()")
    # print(channel)
    channel = channelNameConvert(channel)
    return ports[channel][0].get_values()

def wait_for_edge(channel, edge, child_conn, timeout = -1):
    """Wait for an edge.
    
    channel - gpio channel
    edge - RISING, FALLING or BOTH
    timeout (optional) - time to wait in miliseconds. -1 will wait forever (default)"""
    # print("wait_for_edge()")
    # print(ports)
    channel = channelNameConvert(channel)
    line=ports[channel][0]
    chip=ports[channel][1]
    
    if edge == RISING:
        ev_edge = gpiod.LINE_REQ_EV_RISING_EDGE
    elif edge == FALLING:
        ev_edge = gpiod.LINE_REQ_EV_FALLING_EDGE
    elif edge == BOTH:
        ev_edge = gpiod.LINE_REQ_EV_BOTH_EDGES
    else:
        print("Unknown edge type: " + str(edge))
    
    # Try releasing the line and requesting again
    
    offset = line.to_list()[0].offset()
    line.release()
    line = chip.get_lines([offset])
    
    line.request(consumer=CONSUMER, type=ev_edge)
    x = None
    while not x:
        thread_go = not child_conn.poll()
        if not thread_go:
            return False 
        x = line.event_wait(sec = 1)
    return True

def add_event_detect(channel, edge, callback=None, debounce=0):
    """Enable edge detection events for a particular GPIO channel.
  
    channel      - board pin number.
    edge         - RISING, FALLING or BOTH
    [callback]   - A callback function for the event (optional)
    [bouncetime] - Switch bounce timeout in ms for callback"""
    global threads
    global parent_conn
    channel = channelNameConvert(channel)
    parent_conn_temp, child_conn  = Pipe()
    parent_conn.append(parent_conn_temp)
    process2 = Process(target = run, args = (channel, edge, callback, debounce, child_conn))
    threads.append(process2)
    process2.start()
    return
       
def cleanup():
    """Clean up by resetting all GPIO channels that have been used by 
    this program to INPUT with no pullup/pulldown and no event detection."""
    global ports
    global threads
    i = 0
    for thread in threads:
        parent_conn[i].send(False)
        parent_conn[i].close()
        thread.join()
        thread.close()
        i = i + 1
    for channel, val in ports.items():
        ret = val[0].release()
        if ret:
            print(ret)
        ret = val[1].close()
        if ret:
            print(ret)
        ports={}



""" Pins are:
VDD_ADC = P9_32
GNDA_ADC = P9_34
AIN6 = P9_35
AIN5 = P9_36
AIN4 = P9_33
AIN3 = P9_38
AIN2 = P9_37
AIN1 = P9_40
AIN0 = P9_39 
"""

def readVoltage(name):
	volt_file = "/sys/bus/iio/devices/iio:device0/in_voltage" + name[3] + "_raw"
	file = open(volt_file)
	raw_voltage = file.read()
	return int(raw_voltage)

def convertVoltage(raw_voltage):
	""" Ground is 1
	1.8 is 4095
	"""
	converted_voltage = (raw_voltage/4095)*1.8
	return "%.3f" % converted_voltage
	
